@@ -48,8 +48,12 @@ fn test_print_matches() {
     // heading is required for CI
     let mut printer = make_test_printer().heading(Heading::Always);
     let grep = SgLang::from(SupportLang::Tsx).ast_grep(source);
-    let matches = grep.root().find_all(pattern);
-    printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+    let matches = grep.root().find_all(pattern).collect();
+    let processor = printer.get_processor();
+    let buffer = processor
+      .print_matches(matches, "test.tsx".as_ref())
+      .unwrap();
+    printer.process(buffer).unwrap();
     let expected = source
       .lines()
       .enumerate()
@@ -68,8 +72,12 @@ fn test_print_matches_without_heading() {
   for &(source, pattern, note) in MATCHES_CASES {
     let mut printer = make_test_printer().heading(Heading::Never);
     let grep = SgLang::from(SupportLang::Tsx).ast_grep(source);
-    let matches = grep.root().find_all(pattern);
-    printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+    let matches = grep.root().find_all(pattern).collect();
+    let processor = printer.get_processor();
+    let buffer = processor
+      .print_matches(matches, "test.tsx".as_ref())
+      .unwrap();
+    printer.process(buffer).unwrap();
     // append heading to expected
     let output = source
       .lines()
@@ -108,8 +116,12 @@ rule:
     .pop()
     .unwrap();
     let matcher = rule.get_matcher(&globals).expect("should parse");
-    let matches = grep.root().find_all(&matcher);
-    printer.print_rule(matches, file, &rule).expect("test only");
+    let matches = grep.root().find_all(&matcher).collect();
+    let processor = printer.get_processor();
+    let buffer = processor
+      .print_rule(matches, file, &rule)
+      .expect("test only");
+    printer.process(buffer).expect("test only");
     let text = get_text(&printer);
     assert!(text.contains("test.tsx"), "{note}");
     assert!(text.contains("note[test-id]"), "{note}");
@@ -158,8 +170,14 @@ fn test_print_diffs() {
     let fixer = Fixer::from_str(rewrite, &lang).expect("should work");
     let grep = lang.ast_grep(source);
     let matches = grep.root().find_all(pattern);
-    let diffs = matches.map(|n| Diff::generate(n, &pattern, &fixer));
-    printer.print_diffs(diffs, "test.tsx".as_ref()).unwrap();
+    let diffs = matches
+      .map(|n| Diff::generate(n, &pattern, &fixer))
+      .collect();
+    let buffer = printer
+      .get_processor()
+      .print_diffs(diffs, "test.tsx".as_ref())
+      .unwrap();
+    printer.process(buffer).unwrap();
     assert!(get_text(&printer).contains(rewrite), "{note}");
   }
 }
@@ -173,8 +191,12 @@ fn test_overlap_print_impl(heading: Heading) {
   let mut printer = make_test_printer().heading(heading).context((1, 1));
   let lang = SgLang::from(SupportLang::Tsx);
   let grep = lang.ast_grep(src);
-  let matches = grep.root().find_all("Some($A)");
-  printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+  let matches = grep.root().find_all("Some($A)").collect();
+  let buffer = printer
+    .get_processor()
+    .print_matches(matches, "test.tsx".as_ref())
+    .unwrap();
+  printer.process(buffer).unwrap();
   let text = get_text(&printer);
   // Overlapped match should only print once.
   assert_eq!(text.matches("Some(1)").count(), 1);
@@ -198,8 +220,12 @@ fn test_non_overlap_print_impl(heading: Heading) {
   let mut printer = make_test_printer().heading(heading);
   let lang = SgLang::from(SupportLang::Tsx);
   let grep = lang.ast_grep(src);
-  let matches = grep.root().find_all("Some($A)");
-  printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+  let matches = grep.root().find_all("Some($A)").collect();
+  let buffer = printer
+    .get_processor()
+    .print_matches(matches, "test.tsx".as_ref())
+    .unwrap();
+  printer.process(buffer).unwrap();
   let text = get_text(&printer);
   assert_eq!(text.matches("Some(1)").count(), 1);
   assert!(!text.contains("empty"));
@@ -241,9 +267,11 @@ fix: '{rewrite}'"
     let fixer = matcher.fixer.as_ref().expect("should have fixer");
     let matches = grep.root().find_all(&matcher);
     let diffs = matches.map(|n| (Diff::generate(n, &pattern, fixer), &rule));
-    printer
+    let buffer = printer
+      .get_processor()
       .print_rule_diffs(diffs.collect(), Path::new("test.tsx"))
       .expect("test only");
+    printer.process(buffer).expect("test only");
     let text = get_text(&printer);
     assert!(text.contains("test.tsx"), "{note}");
     assert!(text.contains("note[test-id]"), "{note}");
@@ -267,8 +295,12 @@ fn test_before_after() {
       let mut printer = make_test_printer().context((b, a));
       let lang = SgLang::from(SupportLang::Tsx);
       let grep = lang.ast_grep(src);
-      let matches = grep.root().find_all("Some($A)");
-      printer.print_matches(matches, "test.tsx".as_ref()).unwrap();
+      let matches = grep.root().find_all("Some($A)").collect();
+      let buffer = printer
+        .get_processor()
+        .print_matches(matches, "test.tsx".as_ref())
+        .unwrap();
+      printer.process(buffer).unwrap();
       let text = get_text(&printer);
       // Overlapped match should only print once.
       assert!(text.contains("Some(match)"));
