@@ -1,26 +1,16 @@
-use super::{Diff, Printer};
+use super::{Diff, NodeMatch, PrintProcessor, Printer};
 use crate::lang::SgLang;
 use crate::utils;
 use crate::utils::ErrorContext as EC;
 
 use anyhow::{Context, Result};
 use ast_grep_config::RuleConfig;
-use ast_grep_core::{NodeMatch as SgNodeMatch, StrDoc};
 use codespan_reporting::files::SimpleFile;
 
-type NodeMatch<'a, L> = SgNodeMatch<'a, StrDoc<L>>;
-
 use std::borrow::Cow;
+use std::ops::Range;
 use std::path::{Path, PathBuf};
 
-// add this macro because neither trait_alias nor type_alias_impl is supported.
-macro_rules! Matches {
-  ($lt: lifetime) => { impl Iterator<Item = NodeMatch<$lt, SgLang>> };
-}
-macro_rules! Diffs {
-  ($lt: lifetime) => { impl Iterator<Item = Diff<$lt>> };
-}
-
 pub struct InteractivePrinter<P: Printer> {
   accept_all: bool,
   from_stdin: bool,
@@ -58,8 +48,8 @@ impl<P: Printer> InteractivePrinter<P> {
     utils::prompt(VIEW_PROMPT, "qe", Some('\n')).expect("cannot fail")
   }
 
-  fn rewrite_action(&self, diffs: Vec<Diff<'_>>, path: &PathBuf) -> Result<()> {
-    if diffs.is_empty() {
+  fn rewrite_action(&self, diffs: Diffs<()>, path: &PathBuf) -> Result<()> {
+    if diffs.contents.is_empty() {
       return Ok(());
     }
     let new_content = apply_rewrite(diffs);
@@ -70,28 +60,20 @@ impl<P: Printer> InteractivePrinter<P> {
       std::fs::write(path, new_content).with_context(|| EC::WriteFile(path.clone()))
     }
   }
-}
 
-impl<P: Printer> Printer for InteractivePrinter<P> {
-  fn print_rule<'a>(
-    &mut self,
-    matches: Matches!('a),
-    file: SimpleFile<Cow<str>, &String>,
-    rule: &RuleConfig<SgLang>,
-  ) -> Result<()> {
+  fn process_highlights(&mut self, highlights: Highlights<P::Processed>) -> Result<()> {
+    let Highlights {
+      path,
+      first_line,
+      inner,
+    } = highlights;
     utils::run_in_alternate_screen(|| {
-      let matches: Vec<_> = matches.collect();
-      let first_match = match matches.first() {
-        Some(n) => n.start_pos().line(),
-        None => return Ok(()),
-      };
-      let file_path = PathBuf::from(file.name().to_string());
-      self.inner.print_rule(matches.into_iter(), file, rule)?;
+      self.inner.process(inner)?;
       let resp = self.prompt_view();
       if resp == 'q' {
         Err(anyhow::anyhow!("Exit interactive editing"))
       } else if resp == 'e' {
-        open_in_editor(&file_path, first_match)?;
+        open_in_editor(&path, first_line)?;
         Ok(())
       } else {
         Ok(())
@@ -99,38 +81,38 @@ impl<P: Printer> Printer for InteractivePrinter<P> {
     })
   }
 
-  fn print_matches<'a>(&mut self, matches: Matches!('a), path: &Path) -> Result<()> {
-    utils::run_in_alternate_screen(|| print_matches_and_confirm_next(self, matches, path))
+  fn process_diffs(&mut self, diffs: Diffs<P::Processed>) -> Result<()> {
+    let path = diffs.path.clone();
+    let (confirmed, all) = process_diffs_interactive(self, diffs)?;
+    self.rewrite_action(confirmed, &path)?;
+    if all {
+      self.accept_all = true;
+    }
+    Ok(())
+  }
+}
+
+impl<P> Printer for InteractivePrinter<P>
+where
+  P: Printer + 'static,
+  P::Processor: Send + 'static,
+{
+  type Processed = Payload<P>;
+  type Processor = InteractiveProcessor<P>;
+
+  fn get_processor(&self) -> Self::Processor {
+    InteractiveProcessor {
+      inner: self.inner.get_processor(),
+    }
   }
 
-  fn print_diffs<'a>(&mut self, diffs: Diffs!('a), path: &Path) -> Result<()> {
-    let path = path.to_path_buf();
-    let (confirmed, all) =
-      print_diffs_interactive(self, &path, diffs.map(|d| (d, None)).collect())?;
-    self.rewrite_action(confirmed, &path)?;
-    if all {
-      self.accept_all = true;
-      // self.accept_all.store(true, Ordering::SeqCst);
+  fn process(&mut self, processed: Self::Processed) -> Result<()> {
+    use InteractivePayload as IP;
+    match processed {
+      IP::Nothing => Ok(()),
+      IP::Highlights(h) => self.process_highlights(h),
+      IP::Diffs(d) => self.process_diffs(d),
     }
-    Ok(())
-  }
-  fn print_rule_diffs(
-    &mut self,
-    diffs: Vec<(Diff<'_>, &RuleConfig<SgLang>)>,
-    path: &Path,
-  ) -> Result<()> {
-    let path = path.to_path_buf();
-    let (confirmed, all) = print_diffs_interactive(
-      self,
-      &path,
-      diffs.into_iter().map(|(d, r)| (d, Some(r))).collect(),
-    )?;
-    self.rewrite_action(confirmed, &path)?;
-    if all {
-      self.accept_all = true;
-      // self.accept_all.store(true, Ordering::SeqCst);
-    }
-    Ok(())
   }
 
   fn after_print(&mut self) -> Result<()> {
@@ -141,50 +123,181 @@ impl<P: Printer> Printer for InteractivePrinter<P> {
   }
 }
 
-fn print_diffs_interactive<'a>(
-  interactive: &mut InteractivePrinter<impl Printer>,
-  path: &Path,
-  diffs: Vec<(Diff<'a>, Option<&RuleConfig<SgLang>>)>,
-) -> Result<(Vec<Diff<'a>>, bool)> {
+pub struct InteractiveDiff<D> {
+  /// string content for the replacement
+  replacement: String,
+  range: Range<usize>,
+  first_line: usize,
+  display: D,
+}
+
+impl<D> InteractiveDiff<D> {
+  fn new(diff: Diff, display: D) -> Self {
+    Self {
+      first_line: diff.node_match.start_pos().line(),
+      replacement: diff.replacement,
+      range: diff.range,
+      display,
+    }
+  }
+}
+
+pub struct Highlights<D> {
+  path: PathBuf,
+  first_line: usize,
+  inner: D,
+}
+
+pub struct Diffs<D> {
+  path: PathBuf,
+  // TODO: this clone is slow
+  old_source: String,
+  contents: Vec<InteractiveDiff<D>>,
+}
+
+pub enum InteractivePayload<D> {
+  Nothing,
+  Highlights(Highlights<D>),
+  Diffs(Diffs<D>),
+}
+
+pub struct InteractiveProcessor<P: Printer> {
+  inner: P::Processor,
+}
+
+pub type Payload<P> = InteractivePayload<<P as Printer>::Processed>;
+
+impl<P> PrintProcessor<Payload<P>> for InteractiveProcessor<P>
+where
+  P: Printer + 'static,
+{
+  fn print_rule(
+    &self,
+    matches: Vec<NodeMatch>,
+    file: SimpleFile<Cow<str>, &String>,
+    rule: &RuleConfig<SgLang>,
+  ) -> Result<Payload<P>> {
+    let Some(first_match) = matches.first() else {
+      return Ok(InteractivePayload::Nothing);
+    };
+    let first_line = first_match.start_pos().line();
+    let path = PathBuf::from(file.name().to_string());
+    let inner = self.inner.print_rule(matches, file, rule)?;
+    let highlights = Highlights {
+      inner,
+      first_line,
+      path,
+    };
+    Ok(InteractivePayload::Highlights(highlights))
+  }
+
+  fn print_matches(&self, matches: Vec<NodeMatch>, path: &Path) -> Result<Payload<P>> {
+    let Some(first_match) = matches.first() else {
+      return Ok(InteractivePayload::Nothing);
+    };
+    let first_line = first_match.start_pos().line();
+    let inner = self.inner.print_matches(matches, path)?;
+    let highlights = Highlights {
+      inner,
+      first_line,
+      path: path.to_path_buf(),
+    };
+    Ok(InteractivePayload::Highlights(highlights))
+  }
+
+  fn print_diffs(&self, diffs: Vec<Diff>, path: &Path) -> Result<Payload<P>> {
+    let old_source = get_old_source(diffs.first());
+    let mut contents = Vec::with_capacity(diffs.len());
+    for diff in diffs {
+      let display = self.inner.print_diffs(vec![diff.clone()], path)?;
+      let content = InteractiveDiff::new(diff, display);
+      contents.push(content);
+    }
+    Ok(InteractivePayload::Diffs(Diffs {
+      path: path.to_path_buf(),
+      old_source,
+      contents,
+    }))
+  }
+  fn print_rule_diffs(
+    &self,
+    diffs: Vec<(Diff<'_>, &RuleConfig<SgLang>)>,
+    path: &Path,
+  ) -> Result<Payload<P>> {
+    let old_source = get_old_source(diffs.first().map(|d| &d.0));
+    let mut contents = Vec::with_capacity(diffs.len());
+    for (diff, rule) in diffs {
+      let display = self
+        .inner
+        .print_rule_diffs(vec![(diff.clone(), rule)], path)?;
+      let content = InteractiveDiff::new(diff, display);
+      contents.push(content);
+    }
+    Ok(InteractivePayload::Diffs(Diffs {
+      path: path.to_path_buf(),
+      old_source,
+      contents,
+    }))
+  }
+}
+
+fn get_old_source(diff: Option<&Diff>) -> String {
+  let Some(node) = diff else {
+    return String::new();
+  };
+  node.get_root_text().to_string()
+}
+
+fn process_diffs_interactive<P: Printer>(
+  interactive: &mut InteractivePrinter<P>,
+  diffs: Diffs<P::Processed>,
+) -> Result<(Diffs<()>, bool)> {
   let mut confirmed = vec![];
   let mut all = interactive.accept_all;
   let mut end = 0;
-  for (diff, rule) in diffs {
+  let path = diffs.path;
+  for diff in diffs.contents {
     if diff.range.start < end {
       continue;
     }
+    let to_confirm = InteractiveDiff {
+      first_line: diff.first_line,
+      range: diff.range.clone(),
+      replacement: diff.replacement.clone(),
+      display: (),
+    };
     let confirm = all || {
-      let (accept_curr, accept_all) =
-        print_diff_and_prompt_action(interactive, path, (diff.clone(), rule))?;
+      let (accept_curr, accept_all) = print_diff_and_prompt_action(interactive, &path, diff)?;
       all = accept_all;
       accept_curr
     };
     if confirm {
-      end = diff.range.end;
-      confirmed.push(diff);
+      end = to_confirm.range.end;
+      confirmed.push(to_confirm);
       interactive.committed_cnt = interactive.committed_cnt.saturating_add(1);
     }
   }
-  Ok((confirmed, all))
+  let diffs = Diffs {
+    path,
+    old_source: diffs.old_source,
+    contents: confirmed,
+  };
+  Ok((diffs, all))
 }
 /// returns if accept_current and accept_all
-fn print_diff_and_prompt_action(
-  interactive: &mut InteractivePrinter<impl Printer>,
+fn print_diff_and_prompt_action<P: Printer>(
+  interactive: &mut InteractivePrinter<P>,
   path: &Path,
-  (diff, rule): (Diff, Option<&RuleConfig<SgLang>>),
+  processed: InteractiveDiff<P::Processed>,
 ) -> Result<(bool, bool)> {
   utils::run_in_alternate_screen(|| {
     let printer = &mut interactive.inner;
-    if let Some(rule) = rule {
-      printer.print_rule_diffs(vec![(diff.clone(), rule)], path)?;
-    } else {
-      printer.print_diffs(std::iter::once(diff.clone()), path)?;
-    }
+    printer.process(processed.display)?;
     match interactive.prompt_edit() {
       'y' => Ok((true, false)),
       'a' => Ok((true, true)),
       'e' => {
-        let pos = diff.node_match.start_pos().line();
+        let pos = processed.first_line;
         open_in_editor(path, pos)?;
         Ok((false, false))
       }
@@ -195,37 +308,11 @@ fn print_diff_and_prompt_action(
   })
 }
 
-fn print_matches_and_confirm_next<'a>(
-  interactive: &mut InteractivePrinter<impl Printer>,
-  matches: Matches!('a),
-  path: &Path,
-) -> Result<()> {
-  let printer = &mut interactive.inner;
-  let matches: Vec<_> = matches.collect();
-  let first_match = match matches.first() {
-    Some(n) => n.start_pos().line(),
-    None => return Ok(()),
-  };
-  printer.print_matches(matches.into_iter(), path)?;
-  let resp = interactive.prompt_view();
-  if resp == 'q' {
-    Err(anyhow::anyhow!("Exit interactive editing"))
-  } else if resp == 'e' {
-    open_in_editor(path, first_match)?;
-    Ok(())
-  } else {
-    Ok(())
-  }
-}
-
-fn apply_rewrite(diffs: Vec<Diff>) -> String {
+fn apply_rewrite(diffs: Diffs<()>) -> String {
   let mut new_content = String::new();
-  let Some(first) = diffs.first() else {
-    return new_content;
-  };
-  let old_content = first.get_root_text();
+  let old_content = diffs.old_source;
   let mut start = 0;
-  for diff in diffs {
+  for diff in diffs.contents {
     let range = diff.range;
     new_content.push_str(&old_content[start..range.start]);
     new_content.push_str(&diff.replacement);
@@ -279,17 +366,31 @@ language: TypeScript
     .unwrap()
   }
 
-  fn make_diffs<'a>(
-    grep: &'a AstGrep<StrDoc<SgLang>>,
+  fn make_diffs(
+    grep: &AstGrep<StrDoc<SgLang>>,
     matcher: impl Matcher<SgLang>,
     fixer: &Fixer<SgLang>,
-  ) -> Vec<Diff<'a>> {
+  ) -> Diffs<()> {
     let root = grep.root();
-    Visitor::new(&matcher)
+    let old_source = root.root().get_text().to_string();
+    let contents = Visitor::new(&matcher)
       .reentrant(false)
       .visit(root)
-      .map(|nm| Diff::generate(nm, &matcher, fixer))
-      .collect()
+      .map(|nm| {
+        let diff = Diff::generate(nm, &matcher, fixer);
+        InteractiveDiff {
+          first_line: 0,
+          range: diff.range,
+          replacement: diff.replacement,
+          display: (),
+        }
+      })
+      .collect();
+    Diffs {
+      old_source,
+      path: PathBuf::new(),
+      contents,
+    }
   }
 
   #[test]
