@@ -211,15 +211,15 @@ impl PathWorker for RunWithInferredLang {
     self.trace.print_file(path, lang)?;
     let matcher = self.arg.build_pattern(lang)?;
     // match sub region
-    let items = if let Some(sub_langs) = lang.injectable_sg_langs() {
-      let sub_matchers = sub_langs.filter_map(|l| {
-        let pattern = self.arg.build_pattern(l).ok()?;
-        Some((l, pattern))
-      });
-      filter_file_pattern(path, lang, Some(matcher), sub_matchers)?
-    } else {
-      filter_file_pattern(path, lang, Some(matcher), std::iter::empty())?
-    };
+    let sub_langs = lang.injectable_sg_langs().into_iter().flatten();
+    let sub_matchers = sub_langs
+      .filter_map(|l| {
+        let maybe_pattern = self.arg.build_pattern(l);
+        maybe_pattern.ok().map(|pattern| (l, pattern))
+      })
+      .collect::<Vec<_>>();
+
+    let items = filter_file_pattern(path, lang, Some(&matcher), &sub_matchers)?;
     let mut ret = Vec::with_capacity(items.len());
     let rewrite_str = self.arg.rewrite.as_ref();
 
@@ -233,7 +233,9 @@ impl PathWorker for RunWithInferredLang {
           eprintln!("╰▻ {e}");
           None
         });
-      let processed = match_one_file::<P>(processor, &unit, &rewrite)?;
+      let Some(processed) = match_one_file(processor, &unit, &rewrite)? else {
+        continue;
+      };
       ret.push(processed);
     }
     Ok(ret)
@@ -288,7 +290,7 @@ impl Worker for RunWithSpecificLang {
 impl PathWorker for RunWithSpecificLang {
   fn build_walk(&self) -> Result<WalkParallel> {
     let lang = self.arg.lang.expect("must present");
-    Ok(self.arg.input.walk_lang(lang))
+    self.arg.input.walk_lang(lang)
   }
   fn get_trace(&self) -> &FileTrace {
     &self.stats.inner
@@ -299,20 +301,23 @@ impl PathWorker for RunWithSpecificLang {
     processor: &P::Processor,
   ) -> Result<Vec<P::Processed>> {
     let arg = &self.arg;
-    let pattern = self.pattern.clone();
+    let pattern = &self.pattern;
     let lang = arg.lang.expect("must present");
     let Some(path_lang) = SgLang::from_path(path) else {
       return Ok(vec![]);
     };
     self.stats.print_file(path, path_lang)?;
-    let filtered = if path_lang == lang {
-      filter_file_pattern(path, lang, Some(pattern), std::iter::empty())?
+    let (root_matcher, sub_matchers) = if path_lang == lang {
+      (Some(pattern), vec![])
     } else {
-      filter_file_pattern(path, path_lang, None, std::iter::once((lang, pattern)))?
+      (None, vec![(lang, pattern.clone())])
     };
+    let filtered = filter_file_pattern(path, path_lang, root_matcher, &sub_matchers)?;
     let mut ret = Vec::with_capacity(filtered.len());
     for unit in filtered {
-      let processed = match_one_file::<P>(processor, &unit, &self.rewrite)?;
+      let Some(processed) = match_one_file(processor, &unit, &self.rewrite)? else {
+        continue;
+      };
       ret.push(processed);
     }
     Ok(ret)
@@ -327,7 +332,10 @@ impl StdInWorker for RunWithSpecificLang {
   ) -> Result<Vec<P::Processed>> {
     let lang = self.arg.lang.expect("must present");
     let grep = lang.ast_grep(src);
-    let matches = grep.root().find_all(&self.pattern);
+    let mut matches = grep.root().find_all(&self.pattern).peekable();
+    if matches.peek().is_none() {
+      return Ok(vec![]);
+    }
     let rewrite = &self.rewrite;
     let path = Path::new("STDIN");
     let processed = if let Some(rewrite) = rewrite {
@@ -339,24 +347,28 @@ impl StdInWorker for RunWithSpecificLang {
     Ok(vec![processed])
   }
 }
-fn match_one_file<P: Printer>(
-  processor: &P::Processor,
+fn match_one_file<T, P: PrintProcessor<T>>(
+  processor: &P,
   match_unit: &MatchUnit<impl Matcher<SgLang>>,
   rewrite: &Option<Fixer<SgLang>>,
-) -> Result<P::Processed> {
+) -> Result<Option<T>> {
   let MatchUnit {
     path,
     grep,
     matcher,
   } = match_unit;
 
-  let matches = grep.root().find_all(matcher);
-  if let Some(rewrite) = rewrite {
-    let diffs = matches.map(|m| Diff::generate(m, matcher, rewrite));
-    processor.print_diffs(diffs.collect(), path)
-  } else {
-    processor.print_matches(matches.collect(), path)
+  let mut matches = grep.root().find_all(matcher).peekable();
+  if matches.peek().is_none() {
+    return Ok(None);
   }
+  let ret = if let Some(rewrite) = rewrite {
+    let diffs = matches.map(|m| Diff::generate(m, matcher, rewrite));
+    processor.print_diffs(diffs.collect(), path)?
+  } else {
+    processor.print_matches(matches.collect(), path)?
+  };
+  Ok(Some(ret))
 }
 
 #[cfg(test)]
