@@ -52,13 +52,13 @@ fn sg(s: &str) -> Result<()> {
 fn test_sg_scan() -> Result<()> {
   let dir = setup()?;
   let config = dir.path().join("sgconfig.yml");
-  let ret = sg(&format!("sg scan -c {}", config.display()));
+  let ret = sg(&format!("ast-grep scan -c {}", config.display()));
   assert!(ret.is_ok());
-  let ret = sg(&format!("sg scan -c={}", config.display()));
+  let ret = sg(&format!("ast-grep scan -c={}", config.display()));
   assert!(ret.is_ok());
-  let ret = sg(&format!("sg scan --config {}", config.display()));
+  let ret = sg(&format!("ast-grep scan --config {}", config.display()));
   assert!(ret.is_ok());
-  let ret = sg(&format!("sg scan --config={}", config.display()));
+  let ret = sg(&format!("ast-grep scan --config={}", config.display()));
   assert!(ret.is_ok());
   drop(dir);
   Ok(())
@@ -67,7 +67,7 @@ fn test_sg_scan() -> Result<()> {
 #[test]
 fn test_sg_rule_off() -> Result<()> {
   let dir = setup()?;
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan"])
     .assert()
@@ -81,7 +81,7 @@ fn test_sg_rule_off() -> Result<()> {
 #[test]
 fn test_sg_scan_inline_rules() -> Result<()> {
   let inline_rules = "{id: test, language: ts, rule: {pattern: console.log($A)}}";
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .args(["scan", "--stdin", "--inline-rules", inline_rules, "--json"])
     .write_stdin("console.log(123)")
     .assert()
@@ -103,7 +103,7 @@ rule: { pattern: None }
 #[test]
 fn test_sg_scan_multiple_rules_in_one_file() -> Result<()> {
   let dir = create_test_files([("rule.yml", MULTI_RULES), ("test.ts", "Some(123) + None")])?;
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "-r", "rule.yml"])
     .assert()
@@ -118,7 +118,7 @@ fn test_sg_scan_multiple_rules_in_one_file() -> Result<()> {
 #[test]
 fn test_sg_scan_py_empty_text() -> Result<()> {
   let inline_rules = "{id: test, language: py, rule: {pattern: None}}";
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .args(["scan", "--stdin", "--inline-rules", inline_rules])
     .write_stdin("\n\n\n\n\nNone")
     .assert()
@@ -132,7 +132,7 @@ fn test_sg_scan_html() -> Result<()> {
     ("rule.yml", RULE1),
     ("test.html", "<script lang=ts>Some(123)</script>"),
   ])?;
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "-r", "rule.yml", "--inspect=summary"])
     .assert()
@@ -151,7 +151,7 @@ fn test_scan_unused_suppression() -> Result<()> {
     ("rules/rule.yml", RULE1),
     ("test.ts", "None(123) // ast-grep-ignore"),
   ])?;
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan"])
     .assert()
@@ -167,25 +167,25 @@ fn test_unused_suppression_only_in_scan() -> Result<()> {
     ("rules/rule.yml", RULE1),
     ("test.ts", "None(123) // ast-grep-ignore"),
   ])?;
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "-r", "rules/rule.yml"])
     .assert()
     .success()
     .stdout(contains("unused-suppression").not());
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "--filter", "on-rule"])
     .assert()
     .success()
     .stdout(contains("unused-suppression").not());
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "--off", "on-rule"])
     .assert()
     .success()
     .stdout(contains("unused-suppression").not());
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "--inline-rules", RULE1])
     .assert()
@@ -201,7 +201,7 @@ fn test_scan_unused_suppression_off() -> Result<()> {
     ("rules/rule.yml", RULE1),
     ("test.ts", "None(123) // ast-grep-ignore"),
   ])?;
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "--off"])
     .assert()
@@ -212,19 +212,19 @@ fn test_scan_unused_suppression_off() -> Result<()> {
 #[test]
 fn test_severity_override() -> Result<()> {
   let dir = setup()?;
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "--error"])
     .assert()
     .failure()
     .stdout(contains("error"));
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "--error=on-rule"])
     .assert()
     .failure()
     .stdout(contains("error"));
-  Command::cargo_bin("sg")?
+  Command::cargo_bin("ast-grep")?
     .current_dir(dir.path())
     .args(["scan", "--error=not-exist"])
     .assert()
