@@ -1,15 +1,51 @@
-use crate::RuleConfig;
+use crate::{RuleConfig, SerializableRule, SerializableRuleConfig, SerializableRuleCore, Severity};
 
 use ast_grep_core::language::Language;
 use ast_grep_core::{AstGrep, Doc, Matcher, Node, NodeMatch};
 
-use bit_set::BitSet;
 use std::collections::{HashMap, HashSet};
 
-pub struct ScanResult<'r, D: Doc> {
-  pub diffs: Vec<(usize, NodeMatch<'r, D>)>,
-  pub matches: HashMap<usize, Vec<NodeMatch<'r, D>>>,
-  pub unused_suppressions: Vec<NodeMatch<'r, D>>,
+pub struct ScanResult<'t, 'r, D: Doc, L: Language> {
+  pub diffs: Vec<(&'r RuleConfig<L>, NodeMatch<'t, D>)>,
+  pub matches: Vec<(&'r RuleConfig<L>, Vec<NodeMatch<'t, D>>)>,
+}
+
+/// store the index to the rule and the matched node
+/// it will be converted to ScanResult by resolving the rule
+struct ScanResultInner<'t, D: Doc> {
+  diffs: Vec<(usize, NodeMatch<'t, D>)>,
+  matches: HashMap<usize, Vec<NodeMatch<'t, D>>>,
+  unused_suppressions: Vec<NodeMatch<'t, D>>,
+}
+
+impl<'t, D: Doc> ScanResultInner<'t, D> {
+  pub fn into_result<'r, L: Language>(
+    self,
+    combined: &CombinedScan<'r, L>,
+    separate_fix: bool,
+  ) -> ScanResult<'t, 'r, D, L> {
+    let mut diffs: Vec<_> = self
+      .diffs
+      .into_iter()
+      .map(|(idx, nm)| (combined.get_rule(idx), nm))
+      .collect();
+    let mut matches: Vec<_> = self
+      .matches
+      .into_iter()
+      .map(|(idx, nms)| (combined.get_rule(idx), nms))
+      .collect();
+    if let Some(rule) = combined.unused_suppression_rule {
+      if separate_fix {
+        diffs.extend(self.unused_suppressions.into_iter().map(|nm| (rule, nm)));
+        diffs.sort_unstable_by_key(|(_, nm)| nm.range().start);
+      } else {
+        let mut supprs = self.unused_suppressions;
+        supprs.sort_unstable_by_key(|nm| nm.range().start);
+        matches.push((rule, supprs));
+      }
+    }
+    ScanResult { diffs, matches }
+  }
 }
 
 struct Suppressions(HashMap<usize, Suppression>);
@@ -79,17 +115,6 @@ impl MaySuppressed<'_> {
 
 const IGNORE_TEXT: &str = "ast-grep-ignore";
 
-pub struct PreScan {
-  hit_set: BitSet,
-  suppressions: Suppressions,
-}
-
-impl PreScan {
-  pub fn is_empty(&self) -> bool {
-    self.hit_set.is_empty() && self.suppressions.0.is_empty()
-  }
-}
-
 /// A struct to group all rules according to their potential kinds.
 /// This can greatly reduce traversal times and skip unmatchable rules.
 /// Rules are referenced by their index in the rules vector.
@@ -97,6 +122,8 @@ pub struct CombinedScan<'r, L: Language> {
   rules: Vec<&'r RuleConfig<L>>,
   /// a vec of vec, mapping from kind to a list of rule index
   kind_rule_mapping: Vec<Vec<usize>>,
+  /// a rule for unused_suppressions
+  unused_suppression_rule: Option<&'r RuleConfig<L>>,
 }
 
 impl<'r, L: Language> CombinedScan<'r, L> {
@@ -123,55 +150,30 @@ impl<'r, L: Language> CombinedScan<'r, L> {
     Self {
       rules,
       kind_rule_mapping: mapping,
+      unused_suppression_rule: None,
     }
   }
 
-  pub fn find<D>(&self, root: &AstGrep<D>) -> PreScan
-  where
-    D: Doc<Lang = L>,
-  {
-    let mut hit = BitSet::new();
-    let mut suppressions = Suppressions(HashMap::new());
-    for node in root.root().dfs() {
-      suppressions.collect(&node);
-      let kind = node.kind_id() as usize;
-      let Some(rule_idx) = self.kind_rule_mapping.get(kind) else {
-        continue;
-      };
-      for &idx in rule_idx {
-        if hit.contains(idx) {
-          continue;
-        }
-        let rule = &self.rules[idx];
-        if rule.matcher.match_node(node.clone()).is_some() {
-          hit.insert(idx);
-        }
-      }
-    }
-    PreScan {
-      hit_set: hit,
-      suppressions,
+  pub fn set_unused_suppression_rule(&mut self, rule: &'r RuleConfig<L>) {
+    if matches!(rule.severity, Severity::Off) {
+      return;
     }
+    self.unused_suppression_rule = Some(rule);
   }
 
-  pub fn scan<'a, D>(
-    &self,
-    root: &'a AstGrep<D>,
-    pre: PreScan,
-    separate_fix: bool,
-  ) -> ScanResult<'a, D>
+  pub fn scan<'a, D>(&self, root: &'a AstGrep<D>, separate_fix: bool) -> ScanResult<'a, '_, D, L>
   where
     D: Doc<Lang = L>,
   {
-    let mut result = ScanResult {
+    let mut result = ScanResultInner {
       diffs: vec![],
       matches: HashMap::new(),
       unused_suppressions: vec![],
     };
-    let PreScan {
-      hit_set,
-      mut suppressions,
-    } = pre;
+    let mut suppressions = Suppressions(HashMap::new());
+    for node in root.root().dfs() {
+      suppressions.collect(&node);
+    }
     let mut suppression_ids = suppressions.suppression_ids();
     let mut suppression_nodes = HashMap::new();
     for node in root.root().dfs() {
@@ -184,9 +186,6 @@ impl<'r, L: Language> CombinedScan<'r, L> {
       };
       let suppression = suppressions.check_suppression(&node);
       for &idx in rule_idx {
-        if !hit_set.contains(idx) {
-          continue;
-        }
         let rule = &self.rules[idx];
         let Some(ret) = rule.matcher.match_node(node.clone()) else {
           continue;
@@ -215,12 +214,37 @@ impl<'r, L: Language> CombinedScan<'r, L> {
         }
       })
       .collect();
-    result
+    result.into_result(self, separate_fix)
   }
 
-  pub fn get_rule(&self, idx: usize) -> &RuleConfig<L> {
+  pub fn get_rule(&self, idx: usize) -> &'r RuleConfig<L> {
     self.rules[idx]
   }
+
+  pub fn unused_config(severity: Severity, lang: L) -> RuleConfig<L> {
+    let rule: SerializableRule = crate::from_str(r#"{"any": []}"#).unwrap();
+    let core = SerializableRuleCore {
+      rule,
+      constraints: None,
+      fix: crate::from_str(r#"''"#).unwrap(),
+      transform: None,
+      utils: None,
+    };
+    let config = SerializableRuleConfig {
+      core,
+      id: "unused-suppression".to_string(),
+      severity,
+      files: None,
+      ignores: None,
+      language: lang,
+      message: "Unused 'ast-grep-ignore' directive.".into(),
+      metadata: None,
+      note: None,
+      rewriters: None,
+      url: None,
+    };
+    RuleConfig::try_from(config, &Default::default()).unwrap()
+  }
 }
 
 fn parse_suppression_set(text: &str) -> Option<HashSet<String>> {
@@ -269,13 +293,11 @@ language: Tsx",
     let rule = create_rule();
     let rules = vec![&rule];
     let scan = CombinedScan::new(rules);
-    let pre = scan.find(&root);
-    assert_eq!(pre.suppressions.0.len(), 4);
-    let scanned = scan.scan(&root, pre, false);
-    let matches = &scanned.matches[&0];
-    assert_eq!(matches.len(), 2);
-    assert_eq!(matches[0].text(), "console.log('no ignore')");
-    assert_eq!(matches[1].text(), "console.log('ignore another')");
+    let scanned = scan.scan(&root, false);
+    let matches = &scanned.matches[0];
+    assert_eq!(matches.1.len(), 2);
+    assert_eq!(matches.1[0].text(), "console.log('no ignore')");
+    assert_eq!(matches.1[1].text(), "console.log('ignore another')");
   }
 
   #[test]
@@ -291,13 +313,11 @@ language: Tsx",
     let rule = create_rule();
     let rules = vec![&rule];
     let scan = CombinedScan::new(rules);
-    let pre = scan.find(&root);
-    assert_eq!(pre.suppressions.0.len(), 4);
-    let scanned = scan.scan(&root, pre, false);
-    let matches = &scanned.matches[&0];
-    assert_eq!(matches.len(), 2);
-    assert_eq!(matches[0].text(), "console.log('no ignore')");
-    assert_eq!(matches[1].text(), "console.log('ignore another')");
+    let scanned = scan.scan(&root, false);
+    let matches = &scanned.matches[0];
+    assert_eq!(matches.1.len(), 2);
+    assert_eq!(matches.1[0].text(), "console.log('no ignore')");
+    assert_eq!(matches.1[1].text(), "console.log('ignore another')");
   }
 
   #[test]
@@ -310,12 +330,12 @@ language: Tsx",
     let root = TypeScript::Tsx.ast_grep(source);
     let rule = create_rule();
     let rules = vec![&rule];
-    let scan = CombinedScan::new(rules);
-    let pre = scan.find(&root);
-    assert_eq!(pre.suppressions.0.len(), 2);
-    let scanned = scan.scan(&root, pre, false);
-    let unused = &scanned.unused_suppressions;
-    assert_eq!(unused.len(), 1);
-    assert_eq!(unused[0].text(), "// ast-grep-ignore: test");
+    let mut scan = CombinedScan::new(rules);
+    scan.set_unused_suppression_rule(&rule);
+    let scanned = scan.scan(&root, false);
+    assert_eq!(scanned.matches.len(), 2);
+    let unused = &scanned.matches[1];
+    assert_eq!(unused.1.len(), 1);
+    assert_eq!(unused.1[0].text(), "// ast-grep-ignore: test");
   }
 }
