@@ -13,7 +13,7 @@ use std::fmt;
 
 // NB StopBy's JsonSchema is changed in xtask/schema.rs
 // revise schema is easier than manually implementation
-#[derive(Serialize, Clone, Default, JsonSchema)]
+#[derive(Clone, Default, JsonSchema)]
 #[serde(rename_all = "camelCase")]
 pub enum SerializableStopBy {
   #[default]
@@ -22,6 +22,13 @@ pub enum SerializableStopBy {
   Rule(SerializableRule),
 }
 
+impl SerializableStopBy {
+  /// String key used for serializing the Neighbor variant
+  const NEIGHBOR_KEY: &str = "neighbor";
+  /// String key used for serializing the End variant
+  const END_KEY: &str = "end";
+}
+
 struct StopByVisitor;
 impl<'de> Visitor<'de> for StopByVisitor {
   type Value = SerializableStopBy;
@@ -34,10 +41,12 @@ impl<'de> Visitor<'de> for StopByVisitor {
     E: de::Error,
   {
     match value {
-      "neighbor" => Ok(SerializableStopBy::Neighbor),
-      "end" => Ok(SerializableStopBy::End),
+      SerializableStopBy::NEIGHBOR_KEY => Ok(SerializableStopBy::Neighbor),
+      SerializableStopBy::END_KEY => Ok(SerializableStopBy::End),
       v => Err(de::Error::custom(format!(
-        "unknown variant `{v}`, expected `neighbor`, `end` or a rule object",
+        "unknown variant `{v}`, expected `{}`, `{}` or a rule object",
+        SerializableStopBy::NEIGHBOR_KEY,
+        SerializableStopBy::END_KEY,
       ))),
     }
   }
@@ -60,6 +69,19 @@ impl<'de> Deserialize<'de> for SerializableStopBy {
   }
 }
 
+impl Serialize for SerializableStopBy {
+  fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+  where
+    S: serde::Serializer,
+  {
+    match self {
+      SerializableStopBy::Neighbor => serializer.serialize_str(SerializableStopBy::NEIGHBOR_KEY),
+      SerializableStopBy::End => serializer.serialize_str(SerializableStopBy::END_KEY),
+      SerializableStopBy::Rule(rule) => rule.serialize(serializer),
+    }
+  }
+}
+
 pub enum StopBy<L: Language> {
   Neighbor,
   End,
@@ -82,7 +104,16 @@ impl<L: Language> StopBy<L> {
   pub fn defined_vars(&self) -> HashSet<&str> {
     match self {
       StopBy::Rule(rule) => rule.defined_vars(),
-      _ => HashSet::new(),
+      StopBy::End => HashSet::new(),
+      StopBy::Neighbor => HashSet::new(),
+    }
+  }
+
+  pub fn verify_util(&self) -> Result<(), RuleSerializeError> {
+    match self {
+      StopBy::Rule(rule) => rule.verify_util(),
+      StopBy::End => Ok(()),
+      StopBy::Neighbor => Ok(()),
     }
   }
 }
@@ -198,4 +229,22 @@ inside:
     let stop_by = parse_stop_by("pattern: $A");
     assert_eq!(stop_by.defined_vars(), ["A"].into_iter().collect());
   }
+
+  #[test]
+  fn test_serialization_deserialization_symmetry() {
+    let stop = to_stop_by("'neighbor'").expect("cannot parse stopBy");
+    let serialized = serde_yaml::to_string(&stop).expect("cannot serialize stopBy");
+    let deserialized = to_stop_by(&serialized).expect("cannot parse stopBy");
+    assert!(matches!(deserialized, SerializableStopBy::Neighbor));
+
+    let stop = to_stop_by("'end'").expect("cannot parse stopBy");
+    let serialized = serde_yaml::to_string(&stop).expect("cannot serialize stopBy");
+    let deserialized = to_stop_by(&serialized).expect("cannot parse stopBy");
+    assert!(matches!(deserialized, SerializableStopBy::End));
+
+    let stop = to_stop_by("kind: some-kind").expect("cannot parse stopBy");
+    let serialized = serde_yaml::to_string(&stop).expect("cannot serialize stopBy");
+    let deserialized = to_stop_by(&serialized).expect("cannot parse stopBy");
+    assert!(matches!(deserialized, SerializableStopBy::Rule(_)));
+  }
 }
