@@ -169,6 +169,8 @@ impl_lang_expando!(Go, language_go, 'µ');
 impl_lang_expando!(Haskell, language_haskell, 'µ');
 // https://github.com/fwcd/tree-sitter-kotlin/pull/93
 impl_lang_expando!(Kotlin, language_kotlin, 'µ');
+// PHP accepts unicode to be used as some name not var name though
+impl_lang_expando!(Php, language_php, 'µ');
 // we can use any char in unicode range [:XID_Start:]
 // https://docs.python.org/3/reference/lexical_analysis.html#identifiers
 // see also [PEP 3131](https://peps.python.org/pep-3131/) for further details.
@@ -178,7 +180,6 @@ impl_lang_expando!(Ruby, language_ruby, 'µ');
 // we can use any char in unicode range [:XID_Start:]
 // https://doc.rust-lang.org/reference/identifiers.html
 impl_lang_expando!(Rust, language_rust, 'µ');
-impl_lang_expando!(Sql, language_sql, '_');
 //https://docs.swift.org/swift-book/documentation/the-swift-programming-language/lexicalstructure/#Identifiers
 impl_lang_expando!(Swift, language_swift, 'µ');
 
@@ -189,7 +190,6 @@ impl_lang!(Java, language_java);
 impl_lang!(JavaScript, language_javascript);
 impl_lang!(Json, language_json);
 impl_lang!(Lua, language_lua);
-impl_lang!(Php, language_php);
 impl_lang!(Scala, language_scala);
 impl_lang!(Tsx, language_tsx);
 impl_lang!(TypeScript, language_typescript);
@@ -219,7 +219,6 @@ pub enum SupportLang {
   Ruby,
   Rust,
   Scala,
-  Sql,
   Swift,
   Tsx,
   TypeScript,
@@ -231,7 +230,7 @@ impl SupportLang {
     use SupportLang::*;
     &[
       Bash, C, Cpp, CSharp, Css, Elixir, Go, Haskell, Html, Java, JavaScript, Json, Kotlin, Lua,
-      Php, Python, Ruby, Rust, Scala, Sql, Swift, Tsx, TypeScript, Yaml,
+      Php, Python, Ruby, Rust, Scala, Swift, Tsx, TypeScript, Yaml,
     ]
   }
 
@@ -331,7 +330,6 @@ impl_aliases! {
   Ruby => &["rb", "ruby"],
   Rust => &["rs", "rust"],
   Scala => &["scala"],
-  Sql => &["sql"],
   Swift => &["swift"],
   TypeScript => &["ts", "typescript"],
   Tsx => &["tsx"],
@@ -376,7 +374,6 @@ macro_rules! execute_lang_method {
       S::Ruby => Ruby.$method($($pname,)*),
       S::Rust => Rust.$method($($pname,)*),
       S::Scala => Scala.$method($($pname,)*),
-      S::Sql => Sql.$method($($pname,)*),
       S::Swift => Swift.$method($($pname,)*),
       S::Tsx => Tsx.$method($($pname,)*),
       S::TypeScript => TypeScript.$method($($pname,)*),
@@ -441,7 +438,6 @@ fn extensions(lang: SupportLang) -> &'static [&'static str] {
     Ruby => &["rb", "rbw", "gemspec"],
     Rust => &["rs"],
     Scala => &["scala", "sc", "sbt"],
-    Sql => &["sql", "pgsql"],
     Swift => &["swift"],
     TypeScript => &["ts", "cts", "mts"],
     Tsx => &["tsx"],
